{% extends "!layout.html" %}

{% block extrahead %}
{% if use_google_analytics|tobool %}
    <script type="text/javascript">
    var _gaq = _gaq || [];
    _gaq.push(['_setAccount', 'UA-37225609-1']);
    _gaq.push(['_trackPageview']);

    (function() {
        var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
        ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
        var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
    })();
    </script>
{% endif %}

{% if (use_twitter|tobool) or (use_media_buttons|tobool) %}
    <script type="text/javascript">
    !function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0];if(!d.getElementById(id)){js=d.createElement(s);
    js.id=id;js.src="http://platform.twitter.com/widgets.js";
    fjs.parentNode.insertBefore(js,fjs);}}(document,"script","twitter-wjs");
    </script>
{% endif %}

{% if use_media_buttons|tobool %}
    <script type="text/javascript">
    (function() {
    var po = document.createElement('script'); po.type = 'text/javascript'; po.async = true;
    po.src = 'https://apis.google.com/js/plusone.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(po, s);
    })();
    </script>
{% endif %}
{% endblock %}


{% block relbar2 %}{% endblock %}

{% block rootrellink %}
        <li><a href="{{ pathto('index') }}">Home</a>&nbsp;|&nbsp;</li>
        <li><a href="{{ pathto('manual') }}">Manual</a>&nbsp;|&nbsp;</li>
        <li><a href="{{ pathto('mne-python') }}">Python</a>&nbsp;|&nbsp;</li>
        <li><a href="{{ pathto('cite') }}">Cite MNE</a>&nbsp;|&nbsp;</li>
        <!-- <li><a href="{{ pathto('search') }}">Search</a></li> -->
{% endblock %}

{% block relbar1 %}
{% if build_dev_html|tobool %}
<div style="background-color: red; color: white; font-weight:bold; text-align: center; padding: 10px; min-width: 910px">
This documentation is for the development version ({{ release }}) - <a href="http://martinos.org/mne/stable">Stable version</a>
</div>
{% endif %}
<div style="background-color: white; text-align: left; padding: 10px 7px 15px 15px; min-width: 910px">
<div style="float: left">
<a href="{{ pathto('index') }}"><img src="{{
pathto("_static/mne_logo.png", 1) }}" border="0" alt="py4sci"/></a>
</div>

<div style="float: right">
<a href="{{ pathto('index') }}"><img src="{{
pathto("_static/institutions.png", 1) }}" border="0" alt="py4sci"/></a>
</div>
<br style="clear:both"/>
</div>
{{ super() }}
{% endblock %}

{# put the sidebar before the body #}
{% block sidebar1 %}{{ sidebar() }}{% endblock %}
{% block sidebar2 %}{% endblock %}