.. mne-python
.. _mne-python: http://mne-tools.github.com/mne-python-intro
.. _`mne-python GitHub`: http://github.com/mne-tools/mne-python
.. _`mne-python sample dataset`: ftp://surfer.nmr.mgh.harvard.edu/pub/data/MNE-sample-data-processed.tar.gz

