"""Non-Linear sparse inverse solvers"""

# Author: Alexandre Gramfort <gramfort@nmr.mgh.harvard.edu>
#
# License: Simplified BSD

from .mxne_inverse import mixed_norm, tf_mixed_norm
from ._gamma_map import gamma_map
