.. This (-*- rst -*-) format file contains commonly used link targets
   and name substitutions.  It may be included in many files,
   therefore it should only contain link targets and name
   substitutions.  Try grepping for "^\.\. _" to find plausible
   candidates for this list.

.. NOTE: reST targets are
   __not_case_sensitive__, so only one target definition is needed for
   nipy, NIPY, Nipy, etc...

.. git stuff
.. _git: http://git-scm.com/
.. _github: http://github.com
.. _GitHub Help: http://help.github.com
.. _msysgit: http://code.google.com/p/msysgit/downloads/list
.. _git-osx-installer: http://code.google.com/p/git-osx-installer/downloads/list
.. _subversion: http://subversion.tigris.org/
.. _git cheat sheet: http://github.com/guides/git-cheat-sheet
.. _pro git book: http://progit.org/
.. _git svn crash course: http://git-scm.com/course/svn.html
.. _learn.github: http://learn.github.com/
.. _network graph visualizer: http://github.com/blog/39-say-hello-to-the-network-graph-visualizer
.. _git user manual: http://schacon.github.com/git/user-manual.html
.. _git tutorial: http://schacon.github.com/git/gittutorial.html
.. _git community book: http://book.git-scm.com/
.. _git ready: http://www.gitready.com/
.. _git casts: http://www.gitcasts.com/
.. _Fernando's git page: http://www.fperez.org/py4science/git.html
.. _git magic: http://www-cs-students.stanford.edu/~blynn/gitmagic/index.html
.. _git concepts: http://www.eecs.harvard.edu/~cduan/technical/git/
.. _git clone: http://schacon.github.com/git/git-clone.html
.. _git checkout: http://schacon.github.com/git/git-checkout.html
.. _git commit: http://schacon.github.com/git/git-commit.html
.. _git push: http://schacon.github.com/git/git-push.html
.. _git pull: http://schacon.github.com/git/git-pull.html
.. _git add: http://schacon.github.com/git/git-add.html
.. _git status: http://schacon.github.com/git/git-status.html
.. _git diff: http://schacon.github.com/git/git-diff.html
.. _git log: http://schacon.github.com/git/git-log.html
.. _git branch: http://schacon.github.com/git/git-branch.html
.. _git remote: http://schacon.github.com/git/git-remote.html
.. _git rebase: http://schacon.github.com/git/git-rebase.html
.. _git config: http://schacon.github.com/git/git-config.html
.. _why the -a flag?: http://www.gitready.com/beginner/2009/01/18/the-staging-area.html
.. _git staging area: http://www.gitready.com/beginner/2009/01/18/the-staging-area.html
.. _tangled working copy problem: http://tomayko.com/writings/the-thing-about-git
.. _git management: http://kerneltrap.org/Linux/Git_Management
.. _linux git workflow: http://www.mail-archive.com/dri-devel@lists.sourceforge.net/msg39091.html
.. _git parable: http://tom.preston-werner.com/2009/05/19/the-git-parable.html
.. _git foundation: http://matthew-brett.github.com/pydagogue/foundation.html
.. _deleting master on github: http://matthew-brett.github.com/pydagogue/gh_delete_master.html
.. _rebase without tears: http://matthew-brett.github.com/pydagogue/rebase_without_tears.html
.. _resolving a merge: http://schacon.github.com/git/user-manual.html#resolving-a-merge
.. _ipython git workflow: http://mail.scipy.org/pipermail/ipython-dev/2010-October/006746.html

.. other stuff
.. _python: http://www.python.org

.. python packages
.. _pep8: http://pypi.python.org/pypi/pep8
.. _pyflakes: http://pypi.python.org/pypi/pyflakes
.. _coverage: https://pypi.python.org/pypi/coverage
.. _nosetests: https://nose.readthedocs.org/en/latest/
.. _mayavi: http://mayavi.sourceforge.net/
.. _nitime: http://nipy.org/nitime/
.. _joblib: https://pypi.python.org/pypi/joblib
.. _scikit-learn: http://scikit-learn.org/stable/
.. _pysurfer: http://pysurfer.github.io/
.. _pyDICOM: https://pypi.python.org/pypi/pydicom/ 
.. _matplotlib: http://matplotlib.org/
.. _sphinx: http://sphinx-doc.org/
.. _pandas: http://pandas.pydata.org/
.. _PIL: https://pypi.python.org/pypi/PIL

.. python editors
.. _spyder: http://spyder-ide.blogspot.com/
.. _anaconda: http://www.continuum.io/downloads
.. _EPD: https://www.enthought.com/products/epd/
.. _textmate: http://macromates.com/
.. _sublime text: http://www.sublimetext.com/

.. mne stuff
.. _mne command line utilities: http://www.nmr.mgh.harvard.edu/martinos/userInfo/data/MNE_register/
.. _mne-scripts: https://github.com/mne-tools/mne-scripts/

.. _Freesurfer: http://surfer.nmr.mgh.harvard.edu/fswiki/DownloadAndInstall/

.. |emdash| unicode:: U+02014

.. vim: ft=rst
